# -*- coding: utf-8 -*-
# @Author: Catofes
# @Date:   2015-08-15
'''
Class to stores everything into a json file.
'''
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)

import json

from future.builtins import open

from .const import Constant
from .singleton import Singleton
from .utils import utf8_data_to_file


class Storage(Singleton):

    def __init__(self):
        '''
        Database stores every info.

        version int
        # if value in file is unequal to value defined in this class.
        # An database update will be applied.
        user dict:
            username str
            key str
        collections list:
            collection_info(dict):
                collection_name str
                collection_type str
                collection_describe str
                collection_songs list:
                    song_id(int)
        songs dict:
            song_id(int) dict:
                song_id int
                artist str
                song_name str
                mp3_url str
                album_name str
                album_id str
                quality str
                lyric str
                tlyric str
        player_info dict:
            player_list list[dict]
            playing_order list[int]
            playing_mode int
            playing_offset int


        :return:
        '''
        if hasattr(self, '_init'):
            return
        self._init = True

        self.database = {
            'user': {
                'username': '',
                'password': '',
                'user_id': '',
                'nickname': '',
            },
            'collections': [],
            'songs': {},
            'player_info': {
                'player_list': [],
                'player_list_type': '',
                'player_list_title': '',
                'playing_order': [],
                'playing_mode': 0,
                'idx': 0,
                'ridx': 0,
                'playing_volume': 60,
            }
        }
        self.storage_path = Constant.storage_path
        self.cookie_path = Constant.cookie_path

    def login(self, username, password, userid, nickname):
        self.database.update(dict(
            username=username,
            password=password,
            user_id=userid,
            nickname=nickname
        ))

    def logout(self):
        self.database['user'] = {
            'username': '',
            'password': '',
            'user_id': '',
            'nickname': '',
        }

    def load(self):
        try:
            with open(self.storage_path, 'r') as f:
                for k, v in json.load(f).items():
                    if isinstance(self.database[k], dict):
                        self.database[k].update(v)
                    else:
                        self.database[k] = v
        except (OSError, KeyError, ValueError) as e:
            pass
        self.save()

    def save(self):
        with open(self.storage_path, 'w') as f:
            data = json.dumps(self.database)
            utf8_data_to_file(f, data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: omi
# @Date:   2014-08-24 21:51:57
'''
网易云音乐 Api
'''
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)

import json
from collections import OrderedDict
from http.cookiejar import LWPCookieJar

import requests
import requests_cache

from .config import Config
from .storage import Storage
from .encrypt import encrypted_request
from . import logger

requests_cache.install_cache('nemcache', expire_after=3600)

log = logger.getLogger(__name__)

# 歌曲榜单地址
TOP_LIST_ALL = {
    0: ['云音乐新歌榜', '3779629'],
    1: ['云音乐热歌榜', '3778678'],
    2: ['网易原创歌曲榜', '2884035'],
    3: ['云音乐飙升榜', '19723756'],
    4: ['云音乐电音榜', '10520166'],
    5: ['UK排行榜周榜', '180106'],
    6: ['美国Billboard周榜', '60198'],
    7: ['KTV嗨榜', '21845217'],
    8: ['iTunes榜', '11641012'],
    9: ['Hit FM Top榜', '120001'],
    10: ['日本Oricon周榜', '60131'],
    11: ['韩国Melon排行榜周榜', '3733003'],
    12: ['韩国Mnet排行榜周榜', '60255'],
    13: ['韩国Melon原声周榜', '46772709'],
    14: ['中国TOP排行榜(港台榜)', '112504'],
    15: ['中国TOP排行榜(内地榜)', '64016'],
    16: ['香港电台中文歌曲龙虎榜', '10169002'],
    17: ['华语金曲榜', '4395559'],
    18: ['中国嘻哈榜', '1899724'],
    19: ['法国 NRJ EuroHot 30周榜', '27135204'],
    20: ['台湾Hito排行榜', '112463'],
    21: ['Beatport全球电子舞曲榜', '3812895'],
    22: ['云音乐ACG音乐榜', '71385702'],
    23: ['云音乐嘻哈榜', '991319590']
}


PLAYLIST_CLASSES = OrderedDict([
    ('语种', ['华语', '欧美', '日语', '韩语', '粤语', '小语种']),
    ('风格', ['流行', '摇滚', '民谣', '电子', '舞曲', '说唱', '轻音乐', '爵士', '乡村', 'R&B/Soul', '古典', '民族', '英伦', '金属', '朋克', '蓝调', '雷鬼', '世界音乐', '拉丁', '另类/独立', 'New Age', '古风', '后摇', 'Bossa Nova']),
    ('场景', ['清晨', '夜晚', '学习', '工作', '午休', '下午茶', '地铁', '驾车', '运动', '旅行', '散步', '酒吧']),
    ('情感', ['怀旧', '清新', '浪漫', '性感', '伤感', '治愈', '放松', '孤独', '感动', '兴奋', '快乐', '安静', '思念']),
    ('主题', ['影视原声', 'ACG', '儿童', '校园', '游戏', '70后', '80后', '90后', '网络歌曲', 'KTV', '经典', '翻唱', '吉他', '钢琴', '器乐', '榜单', '00后'])
])

DEFAULT_TIMEOUT = 10

BASE_URL = 'http://music.163.com'


class Parse(object):

    @classmethod
    def _song_url_by_id(cls, sid):
        # 128k
        url = 'http://music.163.com/song/media/outer/url?id={}.mp3'.format(sid)
        quality = 'LD 128k'
        return url, quality

    @classmethod
    def song_url(cls, song):
        if 'url' in song:
            # songs_url resp
            url = song['url']
            if url is None:
                return Parse._song_url_by_id(song['id'])
            br = song['br']
            if br >= 320000:
                quality = 'HD'
            elif br >= 192000:
                quality = 'MD'
            else:
                quality = 'LD'
            return url, '{} {}k'.format(quality, br // 1000)
        else:
            # songs_detail resp
            return Parse._song_url_by_id(song['id'])

    @classmethod
    def song_album(cls, song):
        # 对新老接口进行处理
        if 'al' in song:
            if song['al'] is not None:
                album_name = song['al']['name']
                album_id = song['al']['id']
            else:
                album_name = '未知专辑'
                album_id = ''
        elif 'album' in song:
            if song['album'] is not None:
                album_name = song['album']['name']
                album_id = song['album']['id']
            else:
                album_name = '未知专辑'
                album_id = ''
        else:
            raise ValueError
        return album_name, album_id

    @classmethod
    def song_artist(cls, song):
        artist = ''
        # 对新老接口进行处理
        if 'ar' in song:
            artist = ', '.join([a['name'] for a in song['ar']])
        elif 'artists' in song:
            artist = ', '.join([a['name'] for a in song['artists']])
        else:
            artist = '未知艺术家'

        return artist

    @classmethod
    def songs(cls, songs):
        song_info_list = []
        for song in songs:
            url, quality = Parse.song_url(song)
            if not url:
                continue

            album_name, album_id = Parse.song_album(song)
            song_info = {
                'song_id': song['id'],
                'artist': Parse.song_artist(song),
                'song_name': song['name'],
                'album_name': album_name,
                'album_id': album_id,
                'mp3_url': url,
                'quality': quality
            }
            song_info_list.append(song_info)
        return song_info_list

    @classmethod
    def artists(cls, artists):
        return [{
            'artist_id': artist['id'],
            'artists_name': artist['name'],
            'alias': ''.join(artist['alias'])
        } for artist in artists]

    @classmethod
    def albums(cls, albums):
        return [{
            'album_id': album['id'],
            'albums_name': album['name'],
            'artists_name': album['artist']['name']
        } for album in albums]

    @classmethod
    def playlists(cls, playlists):
        return [{
            'playlist_id': pl['id'],
            'playlist_name': pl['name'],
            'creator_name': pl['creator']['nickname']
        } for pl in playlists]


class NetEase(object):

    def __init__(self):
        self.header = {
            'Accept': '*/*',
            'Accept-Encoding': 'gzip,deflate,sdch',
            'Accept-Language': 'zh-CN,zh;q=0.8,gl;q=0.6,zh-TW;q=0.4',
            'Connection': 'keep-alive',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Host': 'music.163.com',
            'Referer': 'http://music.163.com',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36',
        }

        self.storage = Storage()
        cookie_jar = LWPCookieJar(self.storage.cookie_path)
        cookie_jar.load()
        self.session = requests.Session()
        self.session.cookies = cookie_jar
        for cookie in cookie_jar:
            if cookie.is_expired():
                cookie_jar.clear()
                self.storage.database['user'] = {
                    'username': '',
                    'password': '',
                    'user_id': '',
                    'nickname': '',
                }
                self.storage.save()
                break

    @property
    def toplists(self):
        return [l[0] for l in TOP_LIST_ALL.values()]

    def logout(self):
        self.session.cookies.clear()
        self.storage.database['user'] = {
            'username': '',
            'password': '',
            'user_id': '',
            'nickname': '',
        }
        self.session.cookies.save()
        self.storage.save()

    def _raw_request(self, method, endpoint, data=None):
        if method == 'GET':
            resp = self.session.get(
                endpoint, params=data, headers=self.header, timeout=DEFAULT_TIMEOUT
            )
        elif method == 'POST':
            resp = self.session.post(
                endpoint, data=data, headers=self.header, timeout=DEFAULT_TIMEOUT
            )
        return resp

    def request(self, method, path, params={}, default={'code': -1}):
        endpoint = '{}{}'.format(BASE_URL, path)
        csrf_token = ''
        for cookie in self.session.cookies:
            if cookie.name == '__csrf':
                csrf_token = cookie.value
                break
        params.update({'csrf_token': csrf_token})
        data = default

        params = encrypted_request(params)
        try:
            resp = self._raw_request(method, endpoint, params)
            data = resp.json()
        except requests.exceptions.RequestException as e:
            log.error(e)
        except ValueError as e:
            log.error('Path: {}, response: {}'.format(path, resp.text[:200]))
        finally:
            return data

    def login(self, username, password):
        self.session.cookies.load()
        if username.isdigit():
            path = '/weapi/login/cellphone'
            params = dict(
                phone=username,
                password=password,
                rememberLogin='true',
            )
        else:
            # magic token for login
            # see https://github.com/Binaryify/NeteaseCloudMusicApi/blob/master/router/login.js#L15
            client_token = '1_jVUMqWEPke0/1/Vu56xCmJpo5vP1grjn_SOVVDzOc78w8OKLVZ2JH7IfkjSXqgfmh'
            path = '/weapi/login'
            params = dict(
                username=username,
                password=password,
                rememberLogin='true',
                clientToken=client_token
            )
        data = self.request('POST', path, params)
        self.session.cookies.save()
        return data

    # 每日签到
    def daily_task(self, is_mobile=True):
        path = '/weapi/point/dailyTask'
        params = dict(type=0 if is_mobile else 1)
        return self.request('POST', path, params)

    # 用户歌单
    def user_playlist(self, uid, offset=0, limit=50):
        path = '/weapi/user/playlist'
        params = dict(
            uid=uid,
            offset=offset,
            limit=limit,
            csrf_token=''
        )
        return self.request('POST', path, params).get('playlist', [])

    # 每日推荐歌单
    def recommend_resource(self):
        path = '/weapi/v1/discovery/recommend/resource'
        return self.request('POST', path).get('recommend', [])

    # 私人FM
    def personal_fm(self):
        path = '/weapi/v1/radio/get'
        return self.request('POST', path).get('data', [])

    # like
    def fm_like(self, songid, like=True, time=25, alg='itembased'):
        path = '/weapi/radio/like'
        params = dict(
            alg=alg,
            trackId=songid,
            like='true' if like else 'false',
            time=time
        )
        return self.request('POST', path, params)['code'] == 200

    # FM trash
    def fm_trash(self, songid, time=25, alg='RT'):
        path = '/weapi/radio/trash/add'
        params = dict(
            songId=songid,
            alg=alg,
            time=time,
        )
        return self.request('POST', path, params)['code'] == 200

    # 搜索单曲(1)，歌手(100)，专辑(10)，歌单(1000)，用户(1002) *(type)*
    def search(self, keywords, stype=1, offset=0, total='true', limit=50):
        path = '/weapi/search/get'
        params = dict(
            s=keywords,
            type=stype,
            offset=offset,
            total=total,
            limit=limit
        )
        return self.request('POST', path, params).get('result', {})

    # 新碟上架
    def new_albums(self, offset=0, limit=50):
        path = '/weapi/album/new'
        params = dict(
            area='ALL',
            offset=offset,
            total=True,
            limit=limit,
        )
        return self.request('POST', path, params).get('albums', [])

    # 歌单（网友精选碟） hot||new http://music.163.com/#/discover/playlist/
    def top_playlists(self, category='全部', order='hot', offset=0, limit=50):
        path = '/weapi/playlist/list'
        params = dict(
            cat=category,
            order=order,
            offset=offset,
            total='true',
            limit=limit
        )
        return self.request('POST', path, params).get('playlists', [])

    def playlist_catelogs(self):
        path = '/weapi/playlist/catalogue'
        return self.request('POST', path)

    # 歌单详情
    def playlist_detail(self, playlist_id):
        path = '/weapi/v3/playlist/detail'
        params = dict(
            id=playlist_id,
            total='true',
            limit=1000,
            n=1000,
            offest=0
        )
        return self.request('POST', path, params).get('playlist', {}).get('tracks', [])

    # 热门歌手 http://music.163.com/#/discover/artist/
    def top_artists(self, offset=0, limit=100):
        path = '/weapi/artist/top'
        params = dict(
            offset=offset,
            total=True,
            limit=limit
        )
        return self.request('POST', path, params).get('artists', [])

    # 热门单曲 http://music.163.com/discover/toplist?id=
    def top_songlist(self, idx=0, offset=0, limit=100):
        playlist_id = TOP_LIST_ALL[idx][1]
        return self.playlist_detail(playlist_id)

    # 歌手单曲
    def artists(self, artist_id):
        path = '/weapi/v1/artist/{}'.format(artist_id)
        return self.request('POST', path).get('hotSongs', [])

    def get_artist_album(self, artist_id, offset=0, limit=50):
        path = '/weapi/artist/albums/{}'.format(artist_id)
        params = dict(
            offset=offset,
            total=True,
            limit=limit
        )
        return self.request('POST', path, params).get('hotAlbums', [])

    # album id --> song id set
    def album(self, album_id):
        path = '/weapi/v1/album/{}'.format(album_id)
        return self.request('POST', path).get('songs', [])

    def song_comments(self, music_id, offset=0, total='false', limit=100):
        path = '/weapi/v1/resource/comments/R_SO_4_{}/'.format(music_id)
        params = dict(
            rid=music_id,
            offset=offset,
            total=total,
            limit=limit
        )
        return self.request('POST', path, params)

    # song ids --> song urls ( details )
    def songs_detail(self, ids):
        path = '/weapi/v3/song/detail'
        params = dict(
            c=json.dumps([{'id': _id} for _id in ids]),
            ids=json.dumps(ids),
        )
        return self.request('POST', path, params).get('songs', [])

    def songs_url(self, ids):
        quality = Config().get('music_quality')
        rate_map = {
            0: 320000,
            1: 192000,
            2: 128000
        }

        path = '/weapi/song/enhance/player/url'
        params = dict(
            ids=ids,
            br=rate_map[quality]
        )
        return self.request('POST', path, params).get('data', [])

    # lyric http://music.163.com/api/song/lyric?os=osx&id= &lv=-1&kv=-1&tv=-1
    def song_lyric(self, music_id):
        path = '/weapi/song/lyric'
        params = dict(
            os='osx',
            id=music_id,
            lv=-1,
            kv=-1,
            tv=-1
        )
        lyric = self.request('POST', path, params).get('lrc', {}).get('lyric', [])
        if not lyric:
            return []
        else:
            return lyric.split('\n')

    def song_tlyric(self, music_id):
        path = '/weapi/song/lyric'
        params = dict(
            os='osx',
            id=music_id,
            lv=-1,
            kv=-1,
            tv=-1
        )
        lyric = self.request('POST', path, params).get('tlyric', {}).get('lyric', [])
        if not lyric:
            return []
        else:
            return lyric.split('\n')

    # 今日最热（0）, 本周最热（10），历史最热（20），最新节目（30）
    def djchannels(self, offset=0, limit=50):
        path = '/weapi/djradio/hot/v1'
        params = dict(
            limit=limit,
            offset=offset
        )
        channels = self.request('POST', path, params).get('djRadios', [])
        return channels

    def djprograms(self, radio_id, asc=False, offset=0, limit=50):
        path = '/weapi/dj/program/byradio'
        params = dict(
            asc=asc,
            radioId=radio_id,
            offset=offset,
            limit=limit
        )
        programs = self.request('POST', path, params).get('programs', [])
        return [p['mainSong'] for p in programs]

    # 获取版本
    def get_version(self):
        action = 'https://pypi.org/pypi/NetEase-MusicBox/json'
        try:
            return requests.get(action).json()
        except requests.exceptions.RequestException as e:
            log.error(e)
            return {}

    def dig_info(self, data, dig_type):
        if not data:
            return []
        if dig_type == 'songs' or dig_type == 'fmsongs':
            urls = self.songs_url([s['id'] for s in data])
            for s, u in zip(data, urls):
                s['url'] = u['url']
                s['br'] = u['br']
            return Parse.songs(data)
        elif dig_type == 'artists':
            return Parse.artists(data)

        elif dig_type == 'albums':
            return Parse.albums(data)

        elif dig_type == 'playlists' or dig_type == 'top_playlists':
            return Parse.playlists(data)

        elif dig_type == 'playlist_classes':
            return list(PLAYLIST_CLASSES.keys())

        elif dig_type == 'playlist_class_detail':
            return PLAYLIST_CLASSES[data]
        else:
            raise ValueError('Invalid dig type')



# encoding: UTF-8
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)
import json
import os
from future.builtins import open

from .singleton import Singleton
from .const import Constant
from .utils import utf8_data_to_file


class Config(Singleton):

    def __init__(self):
        if hasattr(self, '_init'):
            return
        self._init = True

        self.path = Constant.config_path
        self.default_config = {
            'version': 8,
            'cache': {
                'value': False,
                'default': False,
                'describe': ('A toggle to enable cache function or not. '
                             'Set value to true to enable it.')
            },
            'mpg123_parameters': {
                'value': [],
                'default': [],
                'describe': 'The additional parameters when mpg123 start.'
            },
            'aria2c_parameters': {
                'value': [],
                'default': [],
                'describe': ('The additional parameters when '
                             'aria2c start to download something.')
            },
            'music_quality': {
                'value': 0,
                'default': 0,
                'describe': ('Select the quality of the music. '
                             'May be useful when network is terrible. '
                             '0 for high quality, 1 for medium and 2 for low.')
            },
            'global_play_pause': {
                'value': '<ctrl><alt>p',
                'default': '<ctrl><alt>p',
                'describe': 'Global keybind for play/pause.'
                            'Uses gtk notation for keybinds.'
            },
            'global_next': {
                'value': '<ctrl><alt>j',
                'default': '<ctrl><alt>j',
                'describe': 'Global keybind for next song.'
                            'Uses gtk notation for keybinds.'
            },
            'global_previous': {
                'value': '<ctrl><alt>k',
                'default': '<ctrl><alt>k',
                'describe': 'Global keybind for previous song.'
                            'Uses gtk notation for keybinds.'
            },
            'notifier': {
                'value': True,
                'default': True,
                'describe': 'Notifier when switching songs.'
            },
            'translation': {
                'value': True,
                'default': True,
                'describe': 'Foreign language lyrics translation.'
            },
            'osdlyrics': {
                'value': False,
                'default': False,
                'describe': 'Desktop lyrics for musicbox.'
            },
            'osdlyrics_transparent': {
                'value': False,
                'default': False,
                'describe': 'Desktop lyrics transparent bg.'
            },
            'osdlyrics_color': {
                'value': [225, 248, 113],
                'default': [225, 248, 113],
                'describe': 'Desktop lyrics RGB Color.'
            },
            'osdlyrics_size': {
                'value': [600, 60],
                'default': [600, 60],
                'describe': 'Desktop lyrics area size.'
            },
            'osdlyrics_font': {
                'value': ['Decorative', 16],
                'default': ['Decorative', 16],
                'describe': 'Desktop lyrics font-family and font-size.'
            },
            'osdlyrics_background': {
                'value': 'rgba(100, 100, 100, 120)',
                'default': 'rgba(100, 100, 100, 120)',
                'describe': 'Desktop lyrics background color.'
            },
            'osdlyrics_on_top': {
                'value': True,
                'default': True,
                'describe': 'Desktop lyrics OnTopHint.'
            },
            'curses_transparency': {
                'value': False,
                'default': False,
                'describe': 'Set true to make curses transparency.'
            }
        }
        self.config = {}
        if not os.path.isfile(self.path):
            self.generate_config_file()

        with open(self.path, 'r') as f:
            try:
                self.config = json.load(f)
            except ValueError:
                self.generate_config_file()

    def generate_config_file(self):
        with open(self.path, 'w') as f:
            utf8_data_to_file(f, json.dumps(self.default_config, indent=2))

    def save_config_file(self):
        with open(self.path, 'w') as f:
            utf8_data_to_file(f, json.dumps(self.config, indent=2))

    def get(self, name):
        if name not in self.config.keys():
            return self.default_config[name]['value']
        return self.config[name]['value']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: omi
# @Date:   2014-08-24 21:51:57
'''
网易云音乐 Menu
'''
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)

import time
import curses
import threading
import sys
import os
import signal
import webbrowser
import locale
from collections import namedtuple

from future.builtins import range, str

from .api import NetEase
from .player import Player
from .ui import Ui
from .osdlyrics import show_lyrics_new_process
from .config import Config
from .utils import notify
from .storage import Storage
from .cache import Cache
from . import logger


locale.setlocale(locale.LC_ALL, '')

log = logger.getLogger(__name__)


def carousel(left, right, x):
    # carousel x in [left, right]
    if x > right:
        return left
    elif x < left:
        return right
    else:
        return x


shortcut = [
    ['j', 'Down      ', '下移'],
    ['k', 'Up        ', '上移'],
    ['h', 'Back      ', '后退'],
    ['l', 'Forward   ', '前进'],
    ['u', 'Prev page ', '上一页'],
    ['d', 'Next page ', '下一页'],
    ['f', 'Search    ', '快速搜索'],
    ['[', 'Prev song ', '上一曲'],
    [']', 'Next song ', '下一曲'],
    [' ', 'Play/Pause', '播放/暂停'],
    ['?', 'Shuffle          ', '手气不错'],
    ['=', 'Volume+          ', '音量增加'],
    ['-', 'Volume-          ', '音量减少'],
    ['m', 'Menu             ', '主菜单'],
    ['p', 'Present/History  ', '当前/历史播放列表'],
    ['i', 'Music Info       ', '当前音乐信息'],
    ['Shift+p', 'Playing Mode     ', '播放模式切换'],
    ['Shift+a', 'Enter album      ', '进入专辑'],
    ['a', 'Add              ', '添加曲目到打碟'],
    ['z', 'DJ list          ', '打碟列表（退出后清空）'],
    ['s', 'Star      ', '添加到本地收藏'],
    ['c', 'Collection', '本地收藏列表'],
    ['r', 'Remove    ', '删除当前条目'],
    ['Shift+j', 'Move Down ', '向下移动当前条目'],
    ['Shift+k', 'Move Up   ', '向上移动当前条目'],
    [',', 'Like      ', '喜爱'],
    ['Shfit+c', 'Cache     ', '缓存歌曲到本地'],
    ['.', 'Trash FM  ', '删除 FM'],
    ['/', 'Next FM   ', '下一 FM'],
    ['q', 'Quit      ', '退出'],
    ['w', 'Quit&Clear', '退出并清除用户信息']
]


class Menu(object):

    def __init__(self):
        self.config = Config()
        self.datatype = 'main'
        self.title = '网易云音乐'
        self.datalist = [
            '排行榜', '艺术家', '新碟上架', '精选歌单', '我的歌单',
            '主播电台', '每日推荐', '私人FM', '搜索', '帮助'
        ]
        self.offset = 0
        self.index = 0
        self.storage = Storage()
        self.storage.load()
        self.collection = self.storage.database['collections']
        self.player = Player()
        self.player.playing_song_changed_callback = self.song_changed_callback
        self.cache = Cache()
        self.ui = Ui()
        self.api = NetEase()
        self.screen = curses.initscr()
        self.screen.keypad(1)
        self.step = 10
        self.stack = []
        self.djstack = []
        self.at_playing_list = False
        self.enter_flag = True
        signal.signal(signal.SIGWINCH, self.change_term)
        signal.signal(signal.SIGINT, self.send_kill)
        self.menu_starts = time.time()
        self.countdown_start = time.time()
        self.countdown = -1
        self.is_in_countdown = False

    @property
    def user(self):
        return self.storage.database['user']

    @property
    def account(self):
        return self.user['username']

    @property
    def md5pass(self):
        return self.user['password']

    @property
    def userid(self):
        return self.user['user_id']

    @property
    def username(self):
        return self.user['nickname']

    def login(self):
        if self.account and self.md5pass:
            account, md5pass = self.account, self.md5pass
        else:
            account, md5pass = self.ui.build_login()

        resp = self.api.login(account, md5pass)
        if resp['code'] == 200:
            userid = resp['account']['id']
            nickname = resp['profile']['nickname']
            self.storage.login(account, md5pass, userid, nickname)
            return True
        else:
            self.storage.logout()
            x = self.ui.build_login_error()
            if x != ord('1'):
                return False
            return self.login()

    def search(self, category):
        self.ui.screen.timeout(-1)
        SearchArg = namedtuple('SearchArg', ['prompt', 'api_type', 'post_process'])
        category_map = {
            'songs': SearchArg(
                '搜索歌曲：', 1,
                lambda datalist: self.api.songs_url([s['id'] for s in datalist])
            ),
            'albums': SearchArg('搜索专辑：', 10, lambda datalist: datalist),
            'artists': SearchArg('搜索艺术家：', 100, lambda datalist: datalist),
            'playlists': SearchArg('搜索网易精选集：', 1000, lambda datalist: datalist)
        }

        prompt, api_type, post_process = category_map[category]
        keyword = self.ui.get_param(prompt)
        if not keyword:
            return []

        data = self.api.search(keyword, api_type)
        if not data:
            return data

        datalist = post_process(data.get(category, []))
        return self.api.dig_info(datalist, category)

    def change_term(self, signum, frame):
        self.ui.screen.clear()
        self.ui.screen.refresh()

    def send_kill(self, signum, fram):
        self.player.stop()
        self.cache.quit()
        self.storage.save()
        curses.endwin()
        sys.exit()

    def update_alert(self, version):
        latest = Menu().check_version()
        if latest != version and latest != 0:
            notify('MusicBox Update is available', 1)
            time.sleep(0.5)
            notify('NetEase-MusicBox installed version:' + version +
                   '\nNetEase-MusicBox latest version:' + latest, 0)

    def check_version(self):
        # 检查更新 && 签到
        try:
            mobile = self.api.daily_task(is_mobile=True)
            pc = self.api.daily_task(is_mobile=False)

            if mobile['code'] == 200:
                notify('移动端签到成功', 1)
            if pc['code'] == 200:
                notify('PC端签到成功', 1)

            data = self.api.get_version()
            return data['info']['version']
        except KeyError as e:
            return 0

    def start_fork(self, version):
        pid = os.fork()
        if pid == 0:
            Menu().update_alert(version)
        else:
            Menu().start()

    def play_pause(self):
        if self.player.is_empty:
            return
        if not self.player.playing_flag:
            self.player.resume()
        else:
            self.player.pause()

    def next_song(self):
        if self.player.is_empty:
            return
        self.player.next()

    def previous_song(self):
        if self.player.is_empty:
            return
        self.player.prev()

    def start(self):
        self.menu_starts = time.time()
        self.ui.build_menu(self.datatype, self.title, self.datalist,
                           self.offset, self.index, self.step, self.menu_starts)
        self.stack.append([
            self.datatype, self.title, self.datalist, self.offset, self.index
        ])

        show_lyrics_new_process()
        while True:
            datatype = self.datatype
            title = self.title
            datalist = self.datalist
            offset = self.offset
            idx = self.index
            step = self.step
            self.screen.timeout(500)
            key = self.screen.getch()
            self.ui.screen.refresh()

            # term resize
            if key == -1:
                self.player.update_size()

            if self.is_in_countdown:
                if time.time() - self.countdown_start > self.countdown:
                    key = ord('q')

            # 退出
            if key == ord('q'):
                break

            # 退出并清除用户信息
            if key == ord('w'):
                self.api.logout()
                break

            # 上移
            elif key == ord('k'):
                # turn page if at beginning
                if idx == offset:
                    if offset == 0:
                        continue
                    self.offset -= step
                    # 移动光标到最后一列
                    self.index = offset - 1
                else:
                    self.index = carousel(offset, min(
                        len(datalist), offset + step) - 1, idx - 1)
                self.menu_starts = time.time()

            # 下移
            elif key == ord('j'):
                # turn page if at end
                if idx == min(len(datalist), offset + step) - 1:
                    if offset + step >= len(datalist):
                        continue
                    self.offset += step
                    # 移动光标到第一列
                    self.index = offset + step
                else:
                    self.index = carousel(offset, min(
                        len(datalist), offset + step) - 1, idx + 1)
                self.menu_starts = time.time()

            # 数字快捷键
            elif ord('0') <= key <= ord('9'):
                idx = key - ord('0')
                self.ui.build_menu(self.datatype, self.title, self.datalist,
                                   self.offset, idx, self.step, self.menu_starts)
                self.ui.build_loading()
                self.dispatch_enter(idx)
                self.index = 0
                self.offset = 0

            # 向上翻页
            elif key == ord('u'):
                if offset == 0:
                    continue
                self.menu_starts = time.time()
                self.offset -= step

                # e.g. 23 - 10 = 13 --> 10
                self.index = (self.index - step) // step * step

            # 向下翻页
            elif key == ord('d'):
                if offset + step >= len(datalist):
                    continue
                self.menu_starts = time.time()
                self.offset += step

                # e.g. 23 + 10 = 33 --> 30
                self.index = (self.index + step) // step * step

            # 前进
            elif key == ord('l') or key == 10:
                self.enter_flag = True
                if len(self.datalist) <= 0:
                    continue
                self.menu_starts = time.time()
                self.ui.build_loading()
                self.dispatch_enter(idx)
                if self.enter_flag is True:
                    self.index = 0
                    self.offset = 0

            # 回退
            elif key == ord('h'):
                # if not main menu
                if len(self.stack) == 1:
                    continue
                self.menu_starts = time.time()
                up = self.stack.pop()
                self.datatype = up[0]
                self.title = up[1]
                self.datalist = up[2]
                self.offset = up[3]
                self.index = up[4]
                self.at_playing_list = False

            # 搜索
            elif key == ord('f'):
                # 8 is the 'search' menu
                self.dispatch_enter(8)

            # 播放下一曲
            elif key == ord(']'):
                self.next_song()

            # 播放上一曲
            elif key == ord('['):
                self.previous_song()

            # 增加音量
            elif key == ord('='):
                self.player.volume_up()

            # 减少音量
            elif key == ord('-'):
                self.player.volume_down()

            # 随机播放
            elif key == ord('?'):
                if len(self.player.info['player_list']) == 0:
                    continue
                self.player.shuffle()

            # 喜爱
            elif key == ord(','):
                return_data = self.request_api(self.api.fm_like,
                                               self.player.playing_id)
                if return_data:
                    song_name = self.player.playing_name
                    notify('%s added successfully!' % song_name, 0)
                else:
                    notify('Adding song failed!', 0)

            # 删除FM
            elif key == ord('.'):
                if self.datatype == 'fmsongs':
                    if len(self.player.info['player_list']) == 0:
                        continue
                    self.player.next()
                    return_data = self.request_api(
                        self.api.fm_trash, self.player.playing_id)
                    if return_data:
                        notify('Deleted successfully!', 0)

            # 下一FM
            elif key == ord('/'):
                if self.datatype == 'fmsongs':
                    if len(self.player.info['player_list']) == 0:
                        continue
                    if self.player.end_callback:
                        self.player.end_callback()

            # 播放、暂停
            elif key == ord(' '):
                if not self.datalist:
                    continue

                if idx < 0 or idx >= len(self.datalist):
                    self.player.info['idx'] = 0

                # If change to a new playing list. Add playing list and play.
                if datatype == 'songs':
                    self.player.new_player_list('songs', self.title,
                                                self.datalist, -1)
                    self.player.end_callback = None
                    self.player.play_or_pause(idx)
                    self.at_playing_list = True
                elif datatype == 'djchannels':
                    self.player.new_player_list('djchannels', self.title,
                                                self.datalist, -1)
                    self.player.end_callback = None
                    self.player.play_or_pause(idx)
                    self.at_playing_list = True
                elif datatype == 'fmsongs':
                    self.player.change_mode(0)
                    self.player.new_player_list('fmsongs', self.title,
                                                self.datalist, -1)
                    self.player.end_callback = self.fm_callback
                    self.player.play_or_pause(idx)
                    self.at_playing_list = True
                else:
                    self.player.play_or_pause(self.player.info['idx'])

            # 加载当前播放列表
            elif key == ord('p'):
                self.show_playing_song()

            # 播放模式切换
            elif key == ord('P'):
                self.player.change_mode()

            # 进入专辑
            elif key == ord('A'):
                if datatype == 'album':
                    continue
                if datatype in ['songs', 'fmsongs']:
                    song_id = datalist[idx]['song_id']
                    album_id = datalist[idx]['album_id']
                    album_name = datalist[idx]['album_name']
                elif self.player.playing_flag:
                    song_id = self.player.playing_id
                    song_info = self.player.songs.get(str(song_id), {})
                    album_id = song_info.get('album_id', '')
                    album_name = song_info.get('album_name', '')
                else:
                    album_id = 0
                if album_id:
                    self.stack.append([datatype, title, datalist, offset, self.index])
                    songs = self.api.album(album_id)
                    self.datatype = 'songs'
                    self.datalist = self.api.dig_info(songs, 'songs')
                    self.title = '网易云音乐 > 专辑 > %s' % album_name
                    for i in range(len(self.datalist)):
                        if self.datalist[i]['song_id'] == song_id:
                            self.offset = i - i % step
                            self.index = i
                            break

            # 添加到打碟歌单
            elif key == ord('a'):
                if datatype == 'songs' and len(datalist) != 0:
                    self.djstack.append(datalist[idx])
                elif datatype == 'artists':
                    pass

            # 加载打碟歌单
            elif key == ord('z'):
                self.stack.append([datatype, title, datalist, offset, self.index])
                self.datatype = 'songs'
                self.title = '网易云音乐 > 打碟'
                self.datalist = self.djstack
                self.offset = 0
                self.index = 0

            # 添加到本地收藏
            elif key == ord('s'):
                if (datatype == 'songs' or
                        datatype == 'djchannels') and len(datalist) != 0:
                    self.collection.append(datalist[idx])
                    notify('Added successfully', 0)

            # 加载本地收藏
            elif key == ord('c'):
                self.stack.append([datatype, title, datalist, offset, self.index])
                self.datatype = 'songs'
                self.title = '网易云音乐 > 本地收藏'
                self.datalist = self.collection
                self.offset = 0
                self.index = 0

            # 从当前列表移除
            elif key == ord('r'):
                if (datatype in ('songs', 'djchannels', 'fmsongs') and
                        len(datalist) != 0):
                    self.datalist.pop(idx)
                    self.index = carousel(offset, min(
                        len(datalist), offset + step) - 1, idx)

            elif key == ord('t'):
                self.countdown_start = time.time()
                countdown = self.ui.build_timing()
                if not countdown.isdigit():
                    notify('The input should be digit')
                    continue

                countdown = int(countdown)
                if countdown > 0:
                    notify('The musicbox will exit in {} minutes'.format(countdown))
                    self.countdown = countdown * 60
                    self.is_in_countdown = True
                else:
                    notify('The timing exit has been canceled')
                    self.is_in_countdown = False

            # 当前项目下移
            elif key == ord('J'):
                if datatype != 'main' and len(datalist) != 0 and idx + 1 != len(self.datalist):
                    self.menu_starts = time.time()
                    song = self.datalist.pop(idx)
                    self.datalist.insert(idx + 1, song)
                    self.index = idx + 1
                    # 翻页
                    if self.index >= offset + step:
                        self.offset = offset + step

            # 当前项目上移
            elif key == ord('K'):
                if datatype != 'main' and len(datalist) != 0 and idx != 0:
                    self.menu_starts = time.time()
                    song = self.datalist.pop(idx)
                    self.datalist.insert(idx - 1, song)
                    self.index = idx - 1
                    # 翻页
                    if self.index < offset:
                        self.offset = offset - step

            elif key == ord('m'):
                if datatype != 'main':
                    self.stack.append([datatype, title, datalist, offset, self.index])
                    self.datatype = self.stack[0][0]
                    self.title = self.stack[0][1]
                    self.datalist = self.stack[0][2]
                    self.offset = 0
                    self.index = 0

            elif key == ord('g'):
                if datatype == 'help':
                    webbrowser.open_new_tab('https://github.com/darknessomi/musicbox')
                else:
                    self.index = 0
                    self.offset = 0

            elif key == ord('G'):
                self.index = len(self.datalist) - 1
                self.offset = self.index - self.index % step

            # 开始下载
            elif key == ord('C'):
                s = self.datalist[idx]
                cache_thread = threading.Thread(
                    target=self.player.cache_song,
                    args=(s['song_id'], s['song_name'], s['artist'], s['mp3_url'])
                )
                cache_thread.start()

            elif key == ord('i'):
                if self.player.playing_id != -1:
                    webbrowser.open_new_tab(
                        'http://music.163.com/song?id={}'.format(self.player.playing_id)
                    )

            self.ui.build_process_bar(
                self.player.current_song,
                self.player.process_location, self.player.process_length,
                self.player.playing_flag, self.player.info['playing_mode']
            )
            self.ui.build_menu(self.datatype, self.title, self.datalist,
                               self.offset, self.index, self.step, self.menu_starts)

        self.player.stop()
        self.cache.quit()
        self.storage.save()
        curses.endwin()

    def dispatch_enter(self, idx):
        # The end of stack
        netease = self.api
        datatype = self.datatype
        title = self.title
        datalist = self.datalist
        offset = self.offset
        index = self.index
        self.stack.append([datatype, title, datalist, offset, index])

        if idx >= len(self.datalist):
            return False

        if datatype == 'main':
            self.choice_channel(idx)

        # 该艺术家的热门歌曲
        elif datatype == 'artists':
            artist_name = datalist[idx]['artists_name']
            artist_id = datalist[idx]['artist_id']

            self.datatype = 'artist_info'
            self.title += ' > ' + artist_name
            self.datalist = [{
                'item': '{}的热门歌曲'.format(artist_name),
                'id': artist_id,
            }, {
                'item': '{}的所有专辑'.format(artist_name),
                'id': artist_id,
            }]

        elif datatype == 'artist_info':
            self.title += ' > ' + datalist[idx]['item']
            artist_id = datalist[0]['id']
            if idx == 0:
                self.datatype = 'songs'
                songs = netease.artists(artist_id)
                self.datalist = netease.dig_info(songs, 'songs')

            elif idx == 1:
                albums = netease.get_artist_album(artist_id)
                self.datatype = 'albums'
                self.datalist = netease.dig_info(albums, 'albums')

        elif datatype == 'djchannels':
            radio_id = datalist[idx]['id']
            programs = netease.djprograms(radio_id)
            self.title += ' > ' + datalist[idx]['name']
            self.datatype = 'songs'
            self.datalist = netease.dig_info(programs, 'songs')

        # 该专辑包含的歌曲
        elif datatype == 'albums':
            album_id = datalist[idx]['album_id']
            songs = netease.album(album_id)
            self.datatype = 'songs'
            self.datalist = netease.dig_info(songs, 'songs')
            self.title += ' > ' + datalist[idx]['albums_name']

        # 精选歌单选项
        elif datatype == 'playlists':
            data = self.datalist[idx]
            self.datatype = data['datatype']
            self.datalist = netease.dig_info(data['callback'](), self.datatype)
            self.title += ' > ' + data['title']

        # 全站置顶歌单包含的歌曲
        elif datatype == 'top_playlists':
            playlist_id = datalist[idx]['playlist_id']
            songs = netease.playlist_detail(playlist_id)
            self.datatype = 'songs'
            self.datalist = netease.dig_info(songs, 'songs')
            self.title += ' > ' + datalist[idx]['playlist_name']

        # 分类精选
        elif datatype == 'playlist_classes':
            # 分类名称
            data = self.datalist[idx]
            self.datatype = 'playlist_class_detail'
            self.datalist = netease.dig_info(data, self.datatype)
            self.title += ' > ' + data

        # 某一分类的详情
        elif datatype == 'playlist_class_detail':
            # 子类别
            data = self.datalist[idx]
            self.datatype = 'top_playlists'
            log.error(data)
            self.datalist = netease.dig_info(netease.top_playlists(data), self.datatype)
            self.title += ' > ' + data

        # 歌曲评论
        elif datatype in ['songs', 'fmsongs']:
            song_id = datalist[idx]['song_id']
            comments = self.api.song_comments(song_id, limit=100)
            try:
                hotcomments = comments['hotComments']
                comcomments = comments['comments']
            except KeyError:
                hotcomments = comcomments = []
            self.datalist = []
            for one_comment in hotcomments:
                self.datalist.append(
                    u'(热评 %s❤️ ️)%s:%s' % (one_comment['likedCount'], one_comment['user']['nickname'],
                                           one_comment['content']))
            for one_comment in comcomments:
                self.datalist.append(one_comment['content'])
            self.datatype = 'comments'
            self.title = '网易云音乐 > 评论:%s' % datalist[idx]['song_name']
            self.offset = 0
            self.index = 0

        # 歌曲榜单
        elif datatype == 'toplists':
            songs = netease.top_songlist(idx)
            self.title += ' > ' + self.datalist[idx]
            self.datalist = netease.dig_info(songs, 'songs')
            self.datatype = 'songs'

        # 搜索菜单
        elif datatype == 'search':
            self.index = 0
            self.offset = 0
            SearchCategory = namedtuple('SearchCategory', ['type', 'title'])
            idx_map = {
                0: SearchCategory('playlists', '精选歌单搜索列表'),
                1: SearchCategory('songs', '歌曲搜索列表'),
                2: SearchCategory('artists', '艺术家搜索列表'),
                3: SearchCategory('albums', '专辑搜索列表')
            }
            self.datatype, self.title = idx_map[idx]
            self.datalist = self.search(self.datatype)
        else:
            self.enter_flag = False

    def show_playing_song(self):
        if self.player.is_empty:
            return

        if not self.at_playing_list:
            self.stack.append([self.datatype, self.title, self.datalist,
                               self.offset, self.index])
            self.at_playing_list = True

        self.datatype = self.player.info['player_list_type']
        self.title = self.player.info['player_list_title']
        self.datalist = [
            self.player.songs[i] for i in self.player.info['player_list']
        ]
        self.index = self.player.info['idx']
        self.offset = self.index // self.step * self.step

    def song_changed_callback(self):
        if self.at_playing_list:
            self.show_playing_song()

    def fm_callback(self):
        log.debug('FM CallBack.')
        data = self.get_new_fm()
        self.player.append_songs(data)
        if self.datatype == 'fmsongs':
            if self.player.is_empty:
                return
            self.datatype = self.player.info['player_list_type']
            self.title = self.player.info['player_list_title']
            self.datalist = []
            for i in self.player.info['player_list']:
                self.datalist.append(self.player.songs[i])
            self.index = self.player.info['idx']
            self.offset = self.index // self.step * self.step
            if not self.player.playing_flag:
                self.player.play_or_pause(self.index)

    def request_api(self, func, *args):
        result = func(*args)
        if result:
            return result
        if not self.login():
            notify('You need to log in')
            return False
        return func(*args)

    def get_new_fm(self):
        data = self.request_api(self.api.personal_fm)
        if not data:
            return []
        return self.api.dig_info(data, 'fmsongs')

    def choice_channel(self, idx):
        self.offset = 0
        self.index = 0

        if idx == 0:
            self.datalist = self.api.toplists
            self.title += ' > 排行榜'
            self.datatype = 'toplists'
        elif idx == 1:
            artists = self.api.top_artists()
            self.datalist = self.api.dig_info(artists, 'artists')
            self.title += ' > 艺术家'
            self.datatype = 'artists'
        elif idx == 2:
            albums = self.api.new_albums()
            self.datalist = self.api.dig_info(albums, 'albums')
            self.title += ' > 新碟上架'
            self.datatype = 'albums'
        elif idx == 3:
            self.datalist = [{
                'title': '全站置顶',
                'datatype': 'top_playlists',
                'callback': self.api.top_playlists
            }, {
                'title': '分类精选',
                'datatype': 'playlist_classes',
                'callback': lambda: []
            }]
            self.title += ' > 精选歌单'
            self.datatype = 'playlists'
        elif idx == 4:
            myplaylist = self.request_api(self.api.user_playlist, self.userid)
            self.datatype = 'top_playlists'
            self.datalist = self.api.dig_info(myplaylist, self.datatype)
            self.title += ' > ' + self.username + ' 的歌单'
        elif idx == 5:
            self.datatype = 'djchannels'
            self.title += ' > 主播电台'
            self.datalist = self.api.djchannels()
        elif idx == 6:
            myplaylist = self.request_api(self.api.recommend_resource)
            self.datatype = 'top_playlists'
            self.title += ' > 每日推荐'
            self.datalist = self.api.dig_info(myplaylist, self.datatype)
        elif idx == 7:
            self.datatype = 'fmsongs'
            self.title += ' > 私人FM'
            self.datalist = self.get_new_fm()
        elif idx == 8:
            self.datatype = 'search'
            self.title += ' > 搜索'
            self.datalist = ['歌曲', '艺术家', '专辑', '网易精选集']
        elif idx == 9:
            self.datatype = 'help'
            self.title += ' > 帮助'
            self.datalist = shortcut



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from hashlib import md5
import unittest

from NEMbox.api import NetEase, Parse


class TestApi(unittest.TestCase):
    def test_api(self):
        api = NetEase()
        ids = [347230, 496619464, 405998841, 28012031]
        print(api.songs_url(ids))
        print(api.songs_detail(ids))
        print(Parse.song_url(api.songs_detail(ids)[0]))
        # user = api.login('example@163.com', md5(b'').hexdigest())
        # user_id = user['account']['id']
        # print(user)
        # api.logout()
        # print(api.user_playlist(3765346))
        # print(api.song_comments(347230))
        # print(api.search('海阔天空')['result']['songs'])
        # print(api.top_songlist()[0])
        # print(Parse.song_url(api.top_songlist()[0]))
        # print(api.djchannels())
        # print(api.search('测', 1000))
        # print(api.album(38721188))
        # print(api.djchannels()[:5])
        # print(api.channel_detail([348289113]))
        # print(api.djprograms(243, True, limit=5))
        # print(api.request('POST', '/weapi/djradio/hot/v1', params=dict(
        #     category='旅途|城市',
        #     limit=5,
        #     offset=0
        # )))
        # print(api.recommend_resource()[0])
        print(api.songs_url([561307346]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: omi
# @Date:   2014-08-24 21:51:57
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)
import logging

from future.builtins import open

from . import const

FILE_NAME = const.Constant.log_path


with open(FILE_NAME, 'a+') as f:
    f.write('#' * 80)
    f.write('\n')


def getLogger(name):
    log = logging.getLogger(name)
    log.setLevel(logging.DEBUG)

    # File output handler
    fh = logging.FileHandler(FILE_NAME)
    fh.setLevel(logging.DEBUG)
    fh.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(name)s:%(lineno)s: %(message)s'))
    log.addHandler(fh)

    return log



# -*- coding: utf-8 -*-
# @Author: Catofes
# @Date:   2015-08-15
'''
Class to cache songs into local storage.
'''
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)
import threading
import subprocess
import os
import signal

from future.builtins import str

from .const import Constant
from .config import Config
from .singleton import Singleton
from .api import NetEase
from . import logger

log = logger.getLogger(__name__)


class Cache(Singleton):

    def __init__(self):
        if hasattr(self, '_init'):
            return
        self._init = True

        self.const = Constant()
        self.config = Config()
        self.download_lock = threading.Lock()
        self.check_lock = threading.Lock()
        self.downloading = []
        self.aria2c = None
        self.wget = None
        self.stop = False
        self.enable = self.config.get('cache')
        self.aria2c_parameters = self.config.get('aria2c_parameters')

    def _is_cache_successful(self):
        def succ(x):
            return x and x.returncode == 0
        return succ(self.aria2c) or succ(self.wget)

    def _kill_all(self):
        def _kill(p):
            if p:
                os.kill(p.pid, signal.SIGKILL)

        _kill(self.aria2c)
        _kill(self.wget)

    def start_download(self):
        check = self.download_lock.acquire(False)
        if not check:
            return False
        while True:
            if self.stop:
                break
            if not self.enable:
                break
            self.check_lock.acquire()
            if len(self.downloading) <= 0:
                self.check_lock.release()
                break
            data = self.downloading.pop()
            self.check_lock.release()
            song_id = data[0]
            song_name = data[1]
            artist = data[2]
            url = data[3]
            onExit = data[4]
            output_path = Constant.download_dir
            output_file = str(artist) + ' - ' + str(song_name) + '.mp3'
            full_path = os.path.join(output_path, output_file)

            new_url = NetEase().songs_url([song_id])[0]['url']
            log.info('Old:{}. New:{}'.format(url, new_url))
            try:
                para = ['aria2c', '--auto-file-renaming=false',
                        '--allow-overwrite=true', '-d', output_path, '-o',
                        output_file, new_url]
                para.extend(self.aria2c_parameters)
                self.aria2c = subprocess.Popen(para,
                                               stdin=subprocess.PIPE,
                                               stdout=subprocess.PIPE,
                                               stderr=subprocess.PIPE)
                self.aria2c.wait()
            except OSError as e:
                log.warning('{}.\tAria2c is unavailable, fall back to wget'.format(e))

                para = ['wget', '-O', full_path, new_url]
                self.wget = subprocess.Popen(para,
                                             stdin=subprocess.PIPE,
                                             stdout=subprocess.PIPE,
                                             stderr=subprocess.PIPE)
                self.wget.wait()

            if self._is_cache_successful():
                log.debug(str(song_id) + ' Cache OK')
                onExit(song_id, full_path)
        self.download_lock.release()

    def add(self, song_id, song_name, artist, url, onExit):
        self.check_lock.acquire()
        self.downloading.append([song_id, song_name, artist, url, onExit])
        self.check_lock.release()

    def quit(self):
        self.stop = True
        try:
            self._kill_all()
        except (AttributeError, OSError) as e:
            log.error(e)
            pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: omi
# @Date:   2014-08-24 21:51:57
'''
网易云音乐 Ui
'''
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)

import hashlib
import re
import curses
import datetime

from future.builtins import range, str, int

from .scrollstring import truelen, scrollstring
from .storage import Storage
from .config import Config

from . import logger
from . import terminalsize

log = logger.getLogger(__name__)

try:
    import dbus
    dbus_activity = True
except ImportError:
    dbus_activity = False
    log.warn('Qt dbus module is not installed.')
    log.warn('Osdlyrics is not available.')


def break_str(s, start, max_len=80):
    length = len(s)
    i, x = 0, max_len
    res = []
    while i < length:
        res.append(s[i:i + max_len])
        i += x
    return '\n{}'.format(' ' * start).join(res)


class Ui(object):

    def __init__(self):
        self.screen = curses.initscr()
        self.screen.timeout(100)  # the screen refresh every 100ms
        # charactor break buffer
        curses.cbreak()
        self.screen.keypad(1)

        curses.start_color()
        if Config().get('curses_transparency'):
            curses.use_default_colors()
            curses.init_pair(1, curses.COLOR_GREEN, -1)
            curses.init_pair(2, curses.COLOR_CYAN, -1)
            curses.init_pair(3, curses.COLOR_RED, -1)
            curses.init_pair(4, curses.COLOR_YELLOW, -1)
        else:
            curses.init_pair(1, curses.COLOR_GREEN, curses.COLOR_BLACK)
            curses.init_pair(2, curses.COLOR_CYAN, curses.COLOR_BLACK)
            curses.init_pair(3, curses.COLOR_RED, curses.COLOR_BLACK)
            curses.init_pair(4, curses.COLOR_YELLOW, curses.COLOR_BLACK)
        # term resize handling
        size = terminalsize.get_terminal_size()
        self.x = max(size[0], 10)
        self.y = max(size[1], 25)
        self.startcol = int(float(self.x) / 5)
        self.indented_startcol = max(self.startcol - 3, 0)
        self.update_space()
        self.lyric = ''
        self.now_lyric = ''
        self.post_lyric = ''
        self.now_lyric_index = 0
        self.tlyric = ''
        self.storage = Storage()
        self.config = Config()
        self.newversion = False

    def addstr(self, *args):
        if len(args) == 1:
            self.screen.addstr(args[0])
        else:
            self.screen.addstr(args[0], args[1], args[2].encode('utf-8'), *args[3:])

    def build_playinfo(self,
                       song_name,
                       artist,
                       album_name,
                       quality,
                       start,
                       pause=False):
        curses.noecho()
        # refresh top 2 line
        self.screen.move(1, 1)
        self.screen.clrtoeol()
        self.screen.move(2, 1)
        self.screen.clrtoeol()
        if pause:
            self.addstr(1, self.indented_startcol,
                        '_ _ z Z Z ' + quality, curses.color_pair(3))
        else:
            self.addstr(1, self.indented_startcol,
                        '♫  ♪ ♫  ♪ ' + quality, curses.color_pair(3))

        self.addstr(
            1, min(self.indented_startcol + 18, self.x - 1),
            song_name + self.space + artist + '  < ' + album_name + ' >',
            curses.color_pair(4))

        self.screen.refresh()

    def build_process_bar(self,
                          song,
                          now_playing,
                          total_length,
                          playing_flag,
                          playing_mode):

        if not song or not playing_flag:
            return
        name, artist = song['song_name'], song['artist']
        lyrics, tlyrics = song.get('lyric', []), song.get('tlyric', [])

        curses.noecho()
        self.screen.move(3, 1)
        self.screen.clrtoeol()
        self.screen.move(4, 1)
        self.screen.clrtoeol()
        self.screen.move(5, 1)
        self.screen.clrtoeol()
        if total_length <= 0:
            total_length = 1
        if now_playing > total_length or now_playing <= 0:
            now_playing = 0
        if now_playing == 0:
            self.now_lyric_index = 0
            self.now_lyric = ''
            self.post_lyric = ''
        process = '['
        for i in range(0, 33):
            if i < now_playing / total_length * 33:
                if (i + 1) > now_playing / total_length * 33:
                    if playing_flag:
                        process += '>'
                        continue
                process += '='
            else:
                process += ' '
        process += '] '

        now = str(datetime.timedelta(seconds=now_playing)).lstrip('0').lstrip(':')
        total = str(datetime.timedelta(seconds=total_length)).lstrip('0').lstrip(':')
        process += '({}/{})'.format(now, total)

        if playing_mode == 0:
            process = '顺序播放 ' + process
        elif playing_mode == 1:
            process = '顺序循环 ' + process
        elif playing_mode == 2:
            process = '单曲循环 ' + process
        elif playing_mode == 3:
            process = '随机播放 ' + process
        elif playing_mode == 4:
            process = '随机循环 ' + process
        else:
            pass
        self.addstr(3, self.startcol - 2, process, curses.color_pair(1))
        if not lyrics:
            self.now_lyric = '暂无歌词 ~>_<~ \n'
            self.post_lyric = ''
            if dbus_activity and self.config.get('osdlyrics'):
                self.now_playing = '{} - {}\n'.format(name, artist)

        else:
            key = now
            index = 0
            for line in lyrics:
                if key in line:
                    # 计算下一句歌词，判断刷新时的歌词和上一次是否相同来进行index计算
                    if not (self.now_lyric == re.sub('\[.*?\]', '', line)):
                        self.now_lyric_index = self.now_lyric_index + 1
                    if index < len(lyrics) - 1:
                        self.post_lyric = lyrics[index + 1]
                    else:
                        self.post_lyric = ''
                    if not tlyrics:
                        self.now_lyric = line
                    else:
                        self.now_lyric = line
                        for tindex, tline in enumerate(tlyrics):
                            if key in tline and self.config.get('translation'):
                                self.now_lyric = tline + ' || ' + self.now_lyric
                                if not (self.post_lyric == '') and tindex < len(tlyrics) - 1:
                                    self.post_lyric = tlyrics[tindex + 1] + ' || ' + self.post_lyric
                                # 此处已经拿到，直接break即可
                                break
                    # 此处已经拿到，直接break即可
                    break
                index += 1
        self.now_lyric = re.sub('\[.*?\]', '', self.now_lyric)
        self.post_lyric = re.sub('\[.*?\]', '', self.post_lyric)
        if dbus_activity and self.config.get('osdlyrics'):
            try:
                bus = dbus.SessionBus().get_object('org.musicbox.Bus', '/')
                # TODO 环境问题，没有试过桌面歌词，此处需要了解的人加个刷界面操作
                if self.now_lyric == '暂无歌词 ~>_<~ \n':
                    bus.refresh_lyrics(self.now_playing,
                                       dbus_interface='local.musicbox.Lyrics')
                else:
                    bus.refresh_lyrics(self.now_lyric,
                                       dbus_interface='local.musicbox.Lyrics')
            except Exception as e:
                log.error(e)
                pass
        # 根据索引计算双行歌词的显示，其中当前歌词颜色为红色，下一句歌词颜色为白色；
        # 当前歌词从下一句歌词刷新颜色变换，所以当前歌词和下一句歌词位置会交替
        if self.now_lyric_index % 2 == 0:
            self.addstr(4, self.startcol - 2, str(self.now_lyric),
                        curses.color_pair(3))
            self.addstr(5, self.startcol + 1, str(self.post_lyric),
                        curses.A_DIM)
        else:
            self.addstr(4, self.startcol - 2, str(self.post_lyric),
                        curses.A_DIM)
            self.addstr(5, self.startcol + 1, str(self.now_lyric),
                        curses.color_pair(3))
        self.screen.refresh()

    def build_loading(self):
        self.addstr(7, self.startcol, '享受高品质音乐，loading...',
                    curses.color_pair(1))
        self.screen.refresh()

    def build_submenu(self, data):
        pass

    # start is the called timestamp of this function
    def build_menu(self, datatype, title, datalist, offset, index, step,
                   start):
        # keep playing info in line 1
        curses.noecho()
        self.screen.move(7, 1)
        self.screen.clrtobot()
        self.addstr(7, self.startcol, title, curses.color_pair(1))

        if len(datalist) == 0:
            self.addstr(8, self.startcol, '这里什么都没有 -，-')
            return self.screen.refresh()

        if datatype == 'main':
            for i in range(offset, min(len(datalist), offset + step)):
                if i == index:
                    self.addstr(i - offset + 9,
                                self.indented_startcol,
                                '-> ' + str(i) + '. ' + datalist[i],
                                curses.color_pair(2))
                else:
                    self.addstr(i - offset + 9, self.startcol,
                                str(i) + '. ' + datalist[i])

        elif datatype == 'songs' or datatype == 'fmsongs':
            iter_range = min(len(datalist), offset + step)
            for i in range(offset, iter_range):
                if isinstance(datalist[i], str):
                    raise ValueError(datalist)
                # this item is focus
                if i == index:
                    self.addstr(i - offset + 8, 0,
                                ' ' * self.startcol)
                    lead = '-> ' + str(i) + '. '
                    self.addstr(i - offset + 8,
                                self.indented_startcol, lead,
                                curses.color_pair(2))
                    name = '{}{}{}  < {} >'.format(
                        datalist[i]['song_name'], self.space,
                        datalist[i]['artist'], datalist[i]['album_name'])

                    # the length decides whether to scoll
                    if truelen(name) < self.x - self.startcol - 1:
                        self.addstr(
                            i - offset + 8,
                            self.indented_startcol + len(lead), name,
                            curses.color_pair(2))
                    else:
                        name = scrollstring(name + '  ', start)
                        self.addstr(
                            i - offset + 8,
                            self.indented_startcol + len(lead), str(name),
                            curses.color_pair(2))
                else:
                    self.addstr(i - offset + 8, 0,
                                ' ' * self.startcol)
                    self.addstr(
                        i - offset + 8, self.startcol,
                        '{}. {}{}{}  < {} >'.format(
                            i, datalist[i]['song_name'], self.space,
                            datalist[i]['artist'],
                            datalist[i]['album_name'])[:int(self.x * 2)])

            self.addstr(iter_range - offset + 8, 0, ' ' * self.x)

        elif datatype == 'comments':
            # 被选中的评论在最下方显示全部字符，其余评论仅显示一行
            for i in range(offset, min(len(datalist), offset + step)):
                maxlength = min(int(1.8 * self.startcol), len(datalist[i]))
                if i == index:
                    self.addstr(
                        20, self.indented_startcol,
                        '-> ' + str(i) + '. ' +
                        break_str(datalist[i], self.indented_startcol, maxlength),
                        curses.color_pair(2))
                else:
                    self.addstr(
                        i - offset + 9, self.startcol,
                        str(i) + '. ' + datalist[i][:maxlength])

        elif datatype == 'artists':
            for i in range(offset, min(len(datalist), offset + step)):
                if i == index:
                    self.addstr(
                        i - offset + 9, self.indented_startcol,
                        '-> ' + str(i) + '. ' + datalist[i]['artists_name'] +
                        self.space + str(datalist[i]['alias']),
                        curses.color_pair(2))
                else:
                    self.addstr(
                        i - offset + 9, self.startcol,
                        str(i) + '. ' + datalist[i]['artists_name'] +
                        self.space + datalist[i][
                            'alias'])

        elif datatype == 'artist_info':
            for i in range(offset, min(len(datalist), offset + step)):
                if i == index:
                    self.addstr(
                        i - offset + 9, self.indented_startcol,
                        '-> ' + str(i) + '. ' + datalist[i]['item'],
                        curses.color_pair(2))
                else:
                    self.addstr(
                        i - offset + 9, self.startcol,
                        str(i) + '. ' + datalist[i]['item'])

        elif datatype == 'albums':
            for i in range(offset, min(len(datalist), offset + step)):
                if i == index:
                    self.addstr(
                        i - offset + 9, self.indented_startcol,
                        '-> ' + str(i) + '. ' + datalist[i]['albums_name'] +
                        self.space + datalist[i]['artists_name'], curses.color_pair(2))
                else:
                    self.addstr(
                        i - offset + 9, self.startcol,
                        str(i) + '. ' + datalist[i]['albums_name'] +
                        self.space + datalist[i][
                            'artists_name'])

        elif datatype == 'playlists':
            for i in range(offset, min(len(datalist), offset + step)):
                if i == index:
                    self.addstr(
                        i - offset + 9, self.indented_startcol,
                        '-> ' + str(i) + '. ' + datalist[i]['title'],
                        curses.color_pair(2))
                else:
                    self.addstr(
                        i - offset + 9, self.startcol,
                        str(i) + '. ' + datalist[i]['title'])

        elif datatype == 'top_playlists':
            for i in range(offset, min(len(datalist), offset + step)):
                if i == index:
                    self.addstr(
                        i - offset + 9, self.indented_startcol, '-> ' +
                        str(i) + '. ' + datalist[i]['playlist_name'] +
                        self.space + datalist[i]['creator_name'],
                        curses.color_pair(2))
                else:
                    self.addstr(
                        i - offset + 9, self.startcol,
                        str(i) + '. ' + datalist[i]['playlist_name'] +
                        self.space + datalist[i][
                            'creator_name'])

        elif datatype in ('toplists', 'playlist_classes', 'playlist_class_detail'):
            for i in range(offset, min(len(datalist), offset + step)):
                if i == index:
                    self.addstr(i - offset + 9,
                                self.indented_startcol,
                                '-> ' + str(i) + '. ' + datalist[i],
                                curses.color_pair(2))
                else:
                    self.addstr(i - offset + 9, self.startcol,
                                str(i) + '. ' + datalist[i])

        elif datatype == 'djchannels':
            for i in range(offset, min(len(datalist), offset + step)):
                if i == index:
                    self.addstr(
                        i - offset + 8, self.indented_startcol,
                        '-> ' + str(i) + '. ' + datalist[i]['name'],
                        curses.color_pair(2))
                else:
                    self.addstr(
                        i - offset + 8, self.startcol,
                        str(i) + '. ' + datalist[i]['name'])

        elif datatype == 'search':
            self.screen.move(6, 1)
            self.screen.clrtobot()
            self.screen.timeout(-1)
            self.addstr(8, self.startcol, '选择搜索类型:',
                        curses.color_pair(1))
            for i in range(offset, min(len(datalist), offset + step)):
                if i == index:
                    self.addstr(
                        i - offset + 10, self.indented_startcol,
                        '-> ' + str(i) + '.' + datalist[i - 1],
                        curses.color_pair(2))
                else:
                    self.addstr(i - offset + 10, self.startcol,
                                str(i) + '.' + datalist[i - 1])
            self.screen.timeout(100)

        elif datatype == 'help':
            for i in range(offset, min(len(datalist), offset + step)):
                if i == index:
                    self.addstr(
                        i - offset + 9, self.indented_startcol,
                        '-> {}. \'{}{}   {}'.format(
                            i, (datalist[i][0] + '\'').ljust(11),
                            datalist[i][1], datalist[i][2]),
                        curses.color_pair(2))
                else:
                    self.addstr(
                        i - offset + 9, self.startcol,
                        '{}. \'{}{}   {}'.format(
                            i, (datalist[i][0] + '\'').ljust(11),
                            datalist[i][1], datalist[i][2]))

            self.addstr(
                20, 6, 'NetEase-MusicBox 基于Python，所有版权音乐来源于网易，本地不做任何保存')
            self.addstr(21, 10,
                        '按 [G] 到 Github 了解更多信息，帮助改进，或者Star表示支持~~')
            self.addstr(22, self.startcol,
                        'Build with love to music by omi')

        self.screen.refresh()

    def build_login(self):
        self.build_login_bar()
        account = self.get_account()
        password = hashlib.md5(self.get_password().encode('utf-8')).hexdigest()
        return account, password

    def build_login_bar(self):
        curses.noecho()
        self.screen.move(4, 1)
        self.screen.clrtobot()
        self.addstr(5, self.startcol, '请输入登录信息(支持手机登录)',
                    curses.color_pair(1))
        self.addstr(8, self.startcol, '账号:', curses.color_pair(1))
        self.addstr(9, self.startcol, '密码:', curses.color_pair(1))
        self.screen.move(8, 24)
        self.screen.refresh()

    def build_login_error(self):
        self.screen.move(4, 1)
        self.screen.timeout(-1)  # disable the screen timeout
        self.screen.clrtobot()
        self.addstr(8, self.startcol, '艾玛，登录信息好像不对呢 (O_O)#',
                    curses.color_pair(1))
        self.addstr(10, self.startcol, '[1] 再试一次')
        self.addstr(11, self.startcol, '[2] 稍后再试')
        self.addstr(14, self.startcol, '请键入对应数字:', curses.color_pair(2))
        self.screen.refresh()
        x = self.screen.getch()
        self.screen.timeout(100)  # restore the screen timeout
        return x

    def build_timing(self):
        self.screen.move(6, 1)
        self.screen.clrtobot()
        self.screen.timeout(-1)
        self.addstr(8, self.startcol, '输入定时时间(min):',
                    curses.color_pair(1))
        self.addstr(11, self.startcol, 'ps:定时时间为整数，输入0代表取消定时退出',
                    curses.color_pair(1))
        self.screen.timeout(-1)  # disable the screen timeout
        curses.echo()
        timing_time = self.screen.getstr(8, self.startcol + 19, 60)
        self.screen.timeout(100)  # restore the screen timeout
        return timing_time

    def get_account(self):
        self.screen.timeout(-1)  # disable the screen timeout
        curses.echo()
        account = self.screen.getstr(8, self.startcol + 6, 60)
        self.screen.timeout(100)  # restore the screen timeout
        return account.decode('utf-8')

    def get_password(self):
        self.screen.timeout(-1)  # disable the screen timeout
        curses.noecho()
        password = self.screen.getstr(9, self.startcol + 6, 60)
        self.screen.timeout(100)  # restore the screen timeout
        return password.decode('utf-8')

    def get_param(self, prompt_string):
        # keep playing info in line 1
        curses.echo()
        self.screen.move(4, 1)
        self.screen.clrtobot()
        self.addstr(5, self.startcol, prompt_string,
                    curses.color_pair(1))
        self.screen.refresh()
        keyword = self.screen.getstr(10, self.startcol, 60)
        return keyword.decode('utf-8').strip()

    def update_size(self):
        # get terminal size
        size = terminalsize.get_terminal_size()
        x = max(size[0], 10)
        y = max(size[1], 25)
        if (x, y) == (self.x, self.y):  # no need to resize
            return
        self.x, self.y = x, y

        # update intendations
        curses.resizeterm(self.y, self.x)
        self.startcol = int(float(self.x) / 5)
        self.indented_startcol = max(self.startcol - 3, 0)
        self.update_space()
        self.screen.clear()
        self.screen.refresh()

    def update_space(self):
        if self.x > 140:
            self.space = '   -   '
        elif self.x > 80:
            self.space = '  -  '
        else:
            self.space = ' - '
        self.screen.refresh()



from .utils import create_dir, create_file
from .const import Constant

create_dir(Constant.conf_dir)
create_dir(Constant.download_dir)
create_file(Constant.storage_path)
create_file(Constant.log_path, default='')
create_file(Constant.cookie_path, default='#LWP-Cookies-2.0\n')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
网易云音乐 Entry
'''
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)
import curses
import traceback
import argparse
import sys

from future.builtins import str

from .menu import Menu
from .__version__ import __version__ as version


def start():
    parser = argparse.ArgumentParser()
    parser.add_argument("-v",
                        "--version",
                        help="show this version and exit",
                        action="store_true")
    args = parser.parse_args()
    if args.version:
        latest = Menu().check_version()
        curses.endwin()
        print('NetEase-MusicBox installed version:' + version)
        if latest != version:
            print('NetEase-MusicBox latest version:' + str(latest))
        sys.exit()

    nembox_menu = Menu()
    try:
        nembox_menu.start_fork(version)
    except (OSError, TypeError, ValueError, KeyError, IndexError):
        # clean up terminal while failed
        curses.echo()
        curses.nocbreak()
        curses.endwin()
        traceback.print_exc()


if __name__ == '__main__':
    start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# utils.py --- utils for musicbox
# Copyright (c) 2015-2016 omi & Contributors
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)

import platform
import subprocess
import os
from collections import OrderedDict

from future.builtins import str


__all__ = [
    'utf8_data_to_file', 'notify', 'uniq', 'create_dir', 'create_file'
]


def mkdir(path):
    try:
        os.mkdir(path)
        return True
    except OSError:
        return False


def create_dir(path):
    if not os.path.exists(path):
        return mkdir(path)

    if os.path.isdir(path):
        return True

    os.remove(path)
    return mkdir(path)


def create_file(path, default='\n'):
    if not os.path.exists(path):
        with open(path, 'w') as f:
            f.write(default)


def uniq(arr):
    return list(OrderedDict.fromkeys(arr).keys())


def utf8_data_to_file(f, data):
    if hasattr(data, 'decode'):
        f.write(data.decode('utf-8'))
    else:
        f.write(data)


def notify_command_osx(msg, msg_type, t=None):
    command = ['/usr/bin/osascript', '-e']
    tpl = "display notification \"{}\" {} with title \"musicbox\""
    sound = 'sound name \"/System/Library/Sounds/Ping.aiff\"' if msg_type else ''
    command.append(tpl.format(msg, sound).encode('utf-8'))
    return command


def notify_command_linux(msg, t=None):
    command = ['/usr/bin/notify-send']
    command.append(msg.encode('utf-8'))
    if t:
        command.extend(['-t', str(t)])
    command.extend(['-h', 'int:transient:1'])
    return command


def notify(msg, msg_type=0, t=None):
    msg = msg.replace('"', '\\"')
    "Show system notification with duration t (ms)"
    if platform.system() == 'Darwin':
        command = notify_command_osx(msg, msg_type, t)
    else:
        command = notify_command_linux(msg, t)

    try:
        subprocess.call(command)
        return True
    except OSError as e:
        return False


if __name__ == "__main__":
    notify("I'm test \"\"quote", msg_type=1, t=1000)
    notify("I'm test 1", msg_type=1, t=1000)
    notify("I'm test 2", msg_type=0, t=1000)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)
import base64
import binascii
import hashlib
import json
import os

from Cryptodome.Cipher import AES
from future.builtins import int, pow

__all__ = ['encrypted_id', 'encrypted_request']

MODULUS = ('00e0b509f6259df8642dbc35662901477df22677ec152b5ff68ace615bb7'
           'b725152b3ab17a876aea8a5aa76d2e417629ec4ee341f56135fccf695280'
           '104e0312ecbda92557c93870114af6c9d05c4f7f0c3685b7a46bee255932'
           '575cce10b424d813cfe4875d3e82047b97ddef52741d546b8e289dc6935b'
           '3ece0462db0a22b8e7')
PUBKEY = '010001'
NONCE = b'0CoJUm6Qyw8W8jud'


# 歌曲加密算法, 基于https://github.com/yanunon/NeteaseCloudMusic
def encrypted_id(id):
    magic = bytearray('3go8&$8*3*3h0k(2)2', 'u8')
    song_id = bytearray(id, 'u8')
    magic_len = len(magic)
    for i, sid in enumerate(song_id):
        song_id[i] = sid ^ magic[i % magic_len]
    m = hashlib.md5(song_id)
    result = m.digest()
    result = base64.b64encode(result).replace(b'/', b'_').replace(b'+', b'-')
    return result.decode('utf-8')


# 登录加密算法, 基于https://github.com/stkevintan/nw_musicbox
def encrypted_request(text):
    # type: (str) -> dict
    data = json.dumps(text).encode('utf-8')
    secret = create_key(16)
    params = aes(aes(data, NONCE), secret)
    encseckey = rsa(secret, PUBKEY, MODULUS)
    return {'params': params, 'encSecKey': encseckey}


def aes(text, key):
    pad = 16 - len(text) % 16
    text = text + bytearray([pad] * pad)
    encryptor = AES.new(key, 2, b'0102030405060708')
    ciphertext = encryptor.encrypt(text)
    return base64.b64encode(ciphertext)


def rsa(text, pubkey, modulus):
    text = text[::-1]
    rs = pow(int(binascii.hexlify(text), 16),
             int(pubkey, 16), int(modulus, 16))
    return format(rs, 'x').zfill(256)


def create_key(size):
    return binascii.hexlify(os.urandom(size))[:16]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: omi
# @Date:   2014-07-15 15:48:27
# @Last Modified by:   omi
# @Last Modified time: 2015-01-30 18:05:08
'''
网易云音乐 Player
'''
# Let's make some noise
from __future__ import (
    print_function, unicode_literals, division, absolute_import
)

import subprocess
import threading
import time
import os
import random


from future.builtins import str

from .ui import Ui
from .storage import Storage
from .api import NetEase
from .cache import Cache
from .config import Config
from .utils import notify

from . import logger

log = logger.getLogger(__name__)


class Player(object):

    MODE_ORDERED = 0
    MODE_ORDERED_LOOP = 1
    MODE_SINGLE_LOOP = 2
    MODE_RANDOM = 3
    MODE_RANDOM_LOOP = 4

    def __init__(self):
        self.config = Config()
        self.ui = Ui()
        self.popen_handler = None
        # flag stop, prevent thread start
        self.playing_flag = False
        self.process_length = 0
        self.process_location = 0
        self.storage = Storage()
        self.cache = Cache()
        self.end_callback = None
        self.playing_song_changed_callback = None
        self.api = NetEase()

    @property
    def info(self):
        return self.storage.database['player_info']

    @property
    def songs(self):
        return self.storage.database['songs']

    @property
    def index(self):
        return self.info['idx']

    @property
    def list(self):
        return self.info['player_list']

    @property
    def order(self):
        return self.info['playing_order']

    @property
    def mode(self):
        return self.info['playing_mode']

    @property
    def is_ordered_mode(self):
        return self.mode == Player.MODE_ORDERED

    @property
    def is_ordered_loop_mode(self):
        return self.mode == Player.MODE_ORDERED_LOOP

    @property
    def is_single_loop_mode(self):
        return self.mode == Player.MODE_SINGLE_LOOP

    @property
    def is_random_mode(self):
        return self.mode == Player.MODE_RANDOM

    @property
    def is_random_loop_mode(self):
        return self.mode == Player.MODE_RANDOM_LOOP

    @property
    def config_notifier(self):
        return self.config.get('notifier')

    @property
    def config_mpg123(self):
        return self.config.get('mpg123_parameters')

    @property
    def current_song(self):
        if not self.songs:
            return {}

        if not self.is_index_valid:
            return {}
        song_id = self.list[self.index]
        return self.songs.get(song_id, {})

    @property
    def playing_id(self):
        return self.current_song['song_id']

    @property
    def playing_name(self):
        return self.current_song['song_name']

    @property
    def is_empty(self):
        return len(self.list) == 0

    @property
    def is_index_valid(self):
        return 0 <= self.index < len(self.list)

    def notify_playing(self):
        if not self.current_song:
            return

        if not self.config_notifier:
            return

        song = self.current_song
        notify('正在播放: {}\n{}-{}'.format(song['song_name'], song['artist'], song['album_name']))

    def notify_copyright_issue(self):
        log.warning('Song {} is unavailable due to copyright issue.'.format(self.playing_id))
        notify('版权限制，无法播放此歌曲')

    def change_mode(self, step=1):
        self.info['playing_mode'] = (self.info['playing_mode'] + step) % 5

    def build_playinfo(self):
        if not self.current_song:
            return

        self.ui.build_playinfo(
            self.current_song['song_name'],
            self.current_song['artist'],
            self.current_song['album_name'],
            self.current_song['quality'],
            time.time(), pause=not self.playing_flag
        )

    def add_songs(self, songs):
        for song in songs:
            song_id = str(song['song_id'])
            self.info['player_list'].append(song_id)
            if song_id in self.songs:
                self.songs[song_id].update(song)
            else:
                self.songs[song_id] = song

    def stop(self):
        if not self.popen_handler:
            return

        self.playing_flag = False
        self.popen_handler.stdin.write(b'Q\n')
        self.popen_handler.stdin.flush()
        self.popen_handler.kill()
        self.popen_handler = None
        # wait process to be killed
        time.sleep(0.01)

    def tune_volume(self, up=0):
        if not self.popen_handler:
            return

        new_volume = self.info['playing_volume'] + up
        if new_volume > 100:
            new_volume = 100
        elif new_volume < 0:
            new_volume = 0

        self.info['playing_volume'] = new_volume
        self.popen_handler.stdin.write(
            'V {}\n'.format(self.info['playing_volume']).encode()
        )
        self.popen_handler.stdin.flush()

    def switch(self):
        if not self.popen_handler:
            return

        self.playing_flag = not self.playing_flag
        self.popen_handler.stdin.write(b'P\n')
        self.popen_handler.stdin.flush()

        self.build_playinfo()

    def run_mpg123(self, on_exit, url):
        para = ['mpg123', '-R'] + self.config_mpg123
        self.popen_handler = subprocess.Popen(
            para,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )

        self.tune_volume()
        self.popen_handler.stdin.write(b'L ' + url.encode('utf-8') + b'\n')
        self.popen_handler.stdin.flush()

        endless_loop_cnt = 0
        while True:
            if not self.popen_handler:
                break

            strout = self.popen_handler.stdout.readline().decode('utf-8').strip()
            if strout[:2] == '@F':
                # playing, update progress
                out = strout.split(' ')
                self.process_location = int(float(out[3]))
                self.process_length = int(float(out[3]) + float(out[4]))
            elif strout[:2] == '@E':
                # error, stop song and move to next
                self.playing_flag = True
                self.notify_copyright_issue()
                break
            elif strout == '@P 0':
                # end, moving to next
                self.playing_flag = True
                break
            elif strout == '':
                endless_loop_cnt += 1
                # 有播放后没有退出，mpg123一直在发送空消息的情况，此处直接终止处理
                if endless_loop_cnt > 100:
                    log.warning('mpg123 error, halt, endless loop and high cpu use, then we kill it')
                    break

        self.stop()
        if self.playing_flag:
            self.next_idx()
            self.replay()

    def download_lyric(self, is_transalted=False):
        key = 'lyric' if not is_transalted else 'tlyric'

        if key not in self.songs[str(self.playing_id)]:
            self.songs[str(self.playing_id)][key] = []

        if len(self.songs[str(self.playing_id)][key]) > 0:
            return

        if not is_transalted:
            lyric = self.api.song_lyric(self.playing_id)
        else:
            lyric = self.api.song_tlyric(self.playing_id)

        self.songs[str(self.playing_id)][key] = lyric

    def download_song(self, song_id, song_name, artist, url):
        def write_path(song_id, path):
            self.songs[str(song_id)]['cache'] = path

        self.cache.add(song_id, song_name, artist, url, write_path)
        self.cache.start_download()

    def start_playing(self, on_exit, args):
        '''
        Runs the given args in subprocess.Popen, and then calls the function
        on_exit when the subprocess completes.
        on_exit is a callable object, and args is a lists/tuple of args
        that would give to subprocess.Popen.
        '''

        if 'cache' in args.keys() and os.path.isfile(args['cache']):
            thread = threading.Thread(target=self.run_mpg123,
                                      args=(on_exit, args['cache']))
        else:
            thread = threading.Thread(target=self.run_mpg123,
                                      args=(on_exit, args['mp3_url']))
            cache_thread = threading.Thread(
                target=self.download_song,
                args=(args['song_id'], args['song_name'], args['artist'], args['mp3_url'])
            )
            cache_thread.start()

        thread.start()
        lyric_download_thread = threading.Thread(target=self.download_lyric)
        lyric_download_thread.start()
        tlyric_download_thread = threading.Thread(target=self.download_lyric, args=(True,))
        tlyric_download_thread.start()
        # returns immediately after the thread starts
        return thread

    def replay(self):
        if not self.is_index_valid:
            self.stop()
            if self.end_callback:
                log.debug('Callback')
                self.end_callback()
            return

        if not self.current_song:
            return

        self.playing_flag = True
        self.build_playinfo()
        self.notify_playing()
        self.start_playing(lambda: 0, self.current_song)

    def shuffle_order(self):
        self.order.clear()
        self.order.extend(self.list)
        random.shuffle(self.order)
        self.info['ridx'] = 0

    def new_player_list(self, type, title, datalist, offset):
        self.info['player_list_type'] = type
        self.info['player_list_title'] = title
        self.info['idx'] = offset
        self.info['player_list'] = []
        self.info['playing_order'] = []
        self.info['ridx'] = 0
        self.add_songs(datalist)

    def append_songs(self, datalist):
        self.add_songs(datalist)

    def play_or_pause(self, idx):
        if self.is_empty:
            return

        # if same playlists && idx --> same song :: pause/resume it
        if self.info['idx'] == idx:
            if not self.popen_handler:
                self.replay()
            else:
                self.switch()
        else:
            self.info['idx'] = idx
            self.stop()
            # start new play
            self.replay()

    def _swap_song(self):
        plist = self.info['playing_order']
        now_songs = plist.index(self.info['idx'])
        plist[0], plist[now_songs] = plist[now_songs], plist[0]

    def _inc_idx(self):
        if self.info['idx'] < len(self.info['player_list']):
            self.info['idx'] += 1

    def _dec_idx(self):
        if self.info['idx'] >= 0:
            self.info['idx'] -= 1

    def _need_to_shuffle(self):
        playing_list = self.info['playing_order']
        ridx = self.info['ridx']
        idx = self.info['idx']
        if ridx >= len(playing_list) or playing_list[ridx] != idx:
            return True
        else:
            return False

    def next_idx(self):
        if not self.is_index_valid:
            return self.stop()
        playlist_len = len(self.info['player_list'])
        playinglist_len = len(self.info['playing_order'])

        # Playing mode. 0 is ordered. 1 is orderde loop.
        # 2 is single song loop. 3 is single random. 4 is random loop
        if self.info['playing_mode'] == 0:
            self._inc_idx()
        elif self.info['playing_mode'] == 1:
            self.info['idx'] = (self.info['idx'] + 1) % playlist_len
        elif self.info['playing_mode'] == 2:
            self.info['idx'] = self.info['idx']
        elif self.info['playing_mode'] == 3 or self.info['playing_mode'] == 4:
            if self._need_to_shuffle():
                self.shuffle_order()
                playinglist_len = len(self.info['playing_order'])
                # When you regenerate playing list
                # you should keep previous song same.
                self._swap_song()

            self.info['ridx'] += 1
            # Out of border
            if self.info['playing_mode'] == 4:
                self.info['ridx'] %= playinglist_len

            if self.info['ridx'] >= playinglist_len:
                self.info['idx'] = playlist_len
            else:
                self.info['idx'] = self.info['playing_order'][self.info['ridx']]
        else:
            self.info['idx'] += 1
        if self.playing_song_changed_callback is not None:
            self.playing_song_changed_callback()

    def next(self):
        self.stop()
        self.next_idx()
        self.replay()

    def prev_idx(self):
        if not self.is_index_valid:
            self.stop()
            return
        playlist_len = len(self.info['player_list'])
        playinglist_len = len(self.info['playing_order'])
        # Playing mode. 0 is ordered. 1 is orderde loop.
        # 2 is single song loop. 3 is single random. 4 is random loop
        if self.info['playing_mode'] == 0:
            self._dec_idx()
        elif self.info['playing_mode'] == 1:
            self.info['idx'] = (self.info['idx'] - 1) % playlist_len
        elif self.info['playing_mode'] == 2:
            self.info['idx'] = self.info['idx']
        elif self.info['playing_mode'] == 3 or self.info['playing_mode'] == 4:
            if self._need_to_shuffle():
                self.shuffle_order()
                playinglist_len = len(self.info['playing_order'])
            self.info['ridx'] -= 1
            if self.info['ridx'] < 0:
                if self.info['playing_mode'] == 3:
                    self.info['ridx'] = 0
                else:
                    self.info['ridx'] %= playinglist_len
            self.info['idx'] = self.info['playing_order'][self.info['ridx']]
        else:
            self.info['idx'] -= 1
        if self.playing_song_changed_callback is not None:
            self.playing_song_changed_callback()

    def prev(self):
        self.stop()
        self.prev_idx()
        self.replay()

    def shuffle(self):
        self.stop()
        self.info['playing_mode'] = 3
        self.shuffle_order()
        self.info['idx'] = self.info['playing_order'][self.info['ridx']]
        self.replay()

    def volume_up(self):
        self.tune_volume(5)

    def volume_down(self):
        self.tune_volume(-5)

    def update_size(self):
        self.ui.update_size()
        self.build_playinfo()

    def cache_song(self, song_id, song_name, artist, song_url):
        def on_exit(song_id, path):
            self.songs[str(song_id)]['cache'] = path
            self.cache.enable = False

        self.cache.enable = True
        self.cache.add(song_id, song_name, artist, song_url, on_exit)
        self.cache.start_download()
